package main

import (
	"encoding/json"
	"fmt"
)

const (
	Large  = iota // large
	Medium        // medium
	Small         // small
)

type Size int

func (s Size) MarshalText() ([]byte, error) {
	switch s {
	case Large:
		return []byte("large"), nil
	case Medium:
		return []byte("medium"), nil
	case Small:
		return []byte("small"), nil
	default:
		return []byte(""), nil
	}
}

func (s *Size) UnmarshalText(bs []byte) error {
	switch string(bs) {
	case "large":
		*s = Large
	case "medium":
		*s = Medium
	case "small":
		*s = Small
	default:
		return fmt.Errorf("unknown: %s", string(bs))
	}
	return nil
}

func main() {
	// 自定义类型的序列化和反序列化
	var size Size = Small
	bs, _ := json.Marshal(size)
	fmt.Println(string(bs)) // "small"

	var s2 Size
	err := json.Unmarshal(bs, &s2)
	if err != nil {
		fmt.Printf(err.Error())
		return
	}
	fmt.Println(s2) // 2

	// 数组
	sizes := []Size{Large, Small, Medium}
	bs, _ = json.Marshal(sizes)
	fmt.Println(string(bs)) // ["large","small","medium"]
	var s3 []Size
	err = json.Unmarshal(bs, &s3)
	if err != nil {
		fmt.Printf(err.Error())
		return
	}
	fmt.Println(s3) // [0 2 1]
}
